package com.gree.mall.manager.logic;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.util.UuidUtils;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.gree.mall.manager.bean.policy.ItfGreeSynDownloadRecBean;
import com.gree.mall.manager.constant.Constant;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.plus.entity.ItfGreeSynDownloadRec;
import com.gree.mall.manager.plus.entity.ItfGreeSynRule;
import com.gree.mall.manager.plus.service.ItfGreeSynDownloadRecService;
import com.gree.mall.manager.plus.service.ItfGreeSynRuleService;
import com.gree.mall.manager.utils.RedisUtil;
import com.gree.mall.manager.utils.http.HttpUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;

import java.util.*;

@Service
@Slf4j
@RequiredArgsConstructor
public class SyncOrderInfoLogic {

    @Value("${gree.V3.gettoken}")
    private String tokenUrl;
    @Value("${gree.check.barcode}")
    private String checkBarcodeUrl;
    @Value("${gree.repair.settle.get}")
    private String getRepairSettleUrl;
    @Value("${gree.repair.settle.do}")
    private String doRepairSettleUrl;
    @Value("${gree.client.id}")
    private String clientId;
    @Value("${gree.V3.client.secert}")
    private String clientSecert;
    @Value("${gree.api.url}")
    private String apiUrl;
    @Value("${gree.gz.appid}")
    private String gzAppId;
    @Value("${gree.gz.appkey}")
    private String gzAppKey;
    @Value("${gree.fs.appid}")
    private String fsAppId;
    @Value("${gree.fs.appkey}")
    private String fsAppKey;
    //    @Value("${gree.cloud.api.url}")
//    private String cloudApiUrl;
    @Value("${gree.V3.api.url}")
    private String v3ApiUrl;
    @Value("${gree.V3.gettoken}")
    private String v3TokenUrl;
    @Value("${gree.V3.client.secert}")
    private String v3ClientSecert;

    @Autowired
    ItfGreeSynRuleService itfGreeSynRuleService;

    @Autowired
    ItfGreeSynDownloadRecService itfGreeSynDownloadRecService;

    @Value("${gree.four.url}")
    private  String greeUrl;

    @Autowired
    RedisUtil redisUtil;

    public void syncGuang() {
        ItfGreeSynRule itfGreeSynRule = itfGreeSynRuleService.lambdaQuery().eq(ItfGreeSynRule::getSalesWebsitNumber, Constant.gree.GUANGZHOUXIAOSI)
                .one();

        Long time = itfGreeSynRule.getLastSynTime().getTime()/1000;

        String accessToken = this.getAccessTokenV3();

        log.info(greeUrl+ Constant.gree.GET_UPDATE_ORDERS+"/"+Constant.gree.GUANGZHOUXIAOSI+"/"+time);

        Map<String, String> params = new HashMap<>();

        Map<String, String> heard = new HashMap<>();
        heard.put("authorization","Bearer " + accessToken);

        String body = HttpUtils.requestGet(greeUrl+ Constant.gree.GET_UPDATE_ORDERS+"/"+Constant.gree.GUANGZHOUXIAOSI+"/"+time, params,heard);

        Map<String,Object> bodyMap = JSON.parseObject(body);

        log.info("【工单同步数据内容】:"+bodyMap.get("data").toString());

        String s = bodyMap.get("data").toString();


        JSONArray jsonArray = JSONUtil.parseArray(s);

        List<ItfGreeSynDownloadRecBean> itfGreeSynDownloadRecBeans = JSONUtil.toList(jsonArray, ItfGreeSynDownloadRecBean.class);

        if (CollectionUtils.isEmpty(itfGreeSynDownloadRecBeans))
            return;

        List<ItfGreeSynDownloadRec> itfGreeSynDownloadRecs = new ArrayList<>();

        Date date = new Date();
        String batch = IdWorker.getIdStr();

        for (ItfGreeSynDownloadRecBean itfGreeSynDownloadRecBean : itfGreeSynDownloadRecBeans) {
            ItfGreeSynDownloadRec itfGreeSynDownloadRec = new ItfGreeSynDownloadRec();
            itfGreeSynDownloadRec.setSynReqTime(date);
            itfGreeSynDownloadRec.setGreeStatSynStatus(0);
            itfGreeSynDownloadRec.setSalesWebsitNumber(itfGreeSynRule.getSalesWebsitNumber());
            itfGreeSynDownloadRec.setDownloadBatchNo(batch);
            itfGreeSynDownloadRec.setWorkOrderId(itfGreeSynDownloadRecBean.getWorkOrderId());
            itfGreeSynDownloadRec.setUpdatedTimestamp(DateUtil.date(Convert.toLong(itfGreeSynDownloadRecBean.getUpdatedTimestamp())));
            itfGreeSynDownloadRec.setOrderNode(itfGreeSynDownloadRecBean.getOrderNode());
            itfGreeSynDownloadRec.setRegionOutletsNo(itfGreeSynDownloadRecBean.getRegionOutletsNo());
            itfGreeSynDownloadRec.setRegionOutletsName(itfGreeSynDownloadRecBean.getRegionOutletsName());



            itfGreeSynDownloadRecs.add(itfGreeSynDownloadRec);
        }

        itfGreeSynDownloadRecService.saveBatch(itfGreeSynDownloadRecs);

    }




    /**
     * 获取总部接口的请求token
     * @return
     * @throws RemoteServiceException
     */
    public String getAccessTokenV3() throws RemoteServiceException {
        String redisKey = Constant.RedisPrefix.TOKEN_GREE_V3;
        String token = (String)redisUtil.get(redisKey);
//        if(token != null){
//            return token;
//        }
        MultiValueMap<String, String> forms= new LinkedMultiValueMap<String, String>();

        forms.put("client_id", Collections.singletonList(clientId));
        forms.put("client_secret", Collections.singletonList(v3ClientSecert));

        forms.put("grant_type", Collections.singletonList("client_credentials"));
        String s = HttpUtils.requestXwww(v3TokenUrl, forms);
        //String s = HttpUtils.requestPostForm(tokenUrl, map, null);
        log.info("【获取总部token】response:{}",s);
        Map<String,Object> bodyMap = JSON.parseObject(s);
        Map<String,Object> map = JSON.parseObject(bodyMap.get("data").toString());

        return (String) map.get("access_token");
    }


    /**
     * 获取总部接口的请求token
     * @return

     */
    public String getAccessToken() throws RemoteServiceException {
        String redisKey = Constant.RedisPrefix.TOKEN_GREE;
        String token = (String)redisUtil.get(redisKey);
        if(token != null){
            return token;
        }
        MultiValueMap<String, String> forms= new LinkedMultiValueMap<String, String>();

        forms.put("client_id", Collections.singletonList(clientId));
        forms.put("client_secret", Collections.singletonList(clientSecert));
        forms.put("grant_type", Collections.singletonList("client_credentials"));
        String s = HttpUtils.requestXwww(tokenUrl, forms);
        //String s = HttpUtils.requestPostForm(tokenUrl, map, null);
        log.info("【获取总部token】response:{}",s);
        JSONObject jsonObject = JSONObject.parseObject(s);
        String accessToken = jsonObject.getString("access_token");
        if(StringUtils.isNotEmpty(accessToken)) {
            redisUtil.set(Constant.RedisPrefix.TOKEN_GREE, accessToken, 2 * 60 * 60);
        }
        return accessToken;
    }

    public void syncFo() {

        ItfGreeSynRule itfGreeSynRule = itfGreeSynRuleService.lambdaQuery().eq(ItfGreeSynRule::getSalesWebsitNumber, Constant.gree.FOSHANXIAOSI)
                .one();

        Long time = itfGreeSynRule.getLastSynTime().getTime()/1000;

        String accessToken = this.getAccessTokenV3();


        String body = HttpRequest.get(greeUrl+ Constant.gree.GET_UPDATE_ORDERS+"/"+Constant.gree.GUANGZHOUXIAOSI+"/"+time)
                .header("authorization","Bearer " + accessToken)
                .execute().body();


        Map<String,Object> bodyMap = JSON.parseObject(body);

        log.info("【工单同步数据内容】:"+bodyMap.get("data").toString());

        JSONArray jsonArray = JSONUtil.parseArray(JSON.parseObject(bodyMap.get("data").toString()));


        List<ItfGreeSynDownloadRecBean> itfGreeSynDownloadRecBeans = JSONUtil.toList(jsonArray, ItfGreeSynDownloadRecBean.class);

        if (CollectionUtils.isEmpty(itfGreeSynDownloadRecBeans))
            return;

        List<ItfGreeSynDownloadRec> itfGreeSynDownloadRecs = new ArrayList<>();

        Date date = new Date();
        String batch = IdWorker.getIdStr();

        for (ItfGreeSynDownloadRecBean itfGreeSynDownloadRecBean : itfGreeSynDownloadRecBeans) {
            ItfGreeSynDownloadRec itfGreeSynDownloadRec = new ItfGreeSynDownloadRec();
            itfGreeSynDownloadRec.setSynReqTime(date);
            itfGreeSynDownloadRec.setGreeStatSynStatus(0);
            itfGreeSynDownloadRec.setSalesWebsitNumber(itfGreeSynRule.getSalesWebsitNumber());
            itfGreeSynDownloadRec.setDownloadBatchNo(batch);
            itfGreeSynDownloadRec.setWorkOrderId(itfGreeSynDownloadRecBean.getWorkOrderId());
            itfGreeSynDownloadRec.setOrderNode(itfGreeSynDownloadRecBean.getOrderNode());
            itfGreeSynDownloadRec.setRegionOutletsNo(itfGreeSynDownloadRecBean.getRegionOutletsNo());
            itfGreeSynDownloadRec.setRegionOutletsName(itfGreeSynDownloadRecBean.getRegionOutletsName());

            itfGreeSynDownloadRecs.add(itfGreeSynDownloadRec);
        }

        itfGreeSynDownloadRecService.saveBatch(itfGreeSynDownloadRecs);
    }

    public void syncDetail() {

        List<ItfGreeSynDownloadRec> itfGreeSynDownloadRecs = itfGreeSynDownloadRecService.lambdaQuery()
                .eq(ItfGreeSynDownloadRec::getGreeStatSynStatus, 0).last("limit 10").list();






    }
}
